import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { BeerConsumptionReport } from './beer-consumption-report';

const url = '/api/beer-consumption-report';
const serviceName = 'BeerConsumptionReportService';

@Injectable({
  providedIn: 'root',
})
export class BeerConsumptionReportService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(startDate: string, finishDate): Observable<BeerConsumptionReport> {
    const options = { params: new HttpParams() };
    if (startDate !== null) {
      options.params = options.params.set('s', startDate);
    }
    if (finishDate !== null) {
      options.params = options.params.set('f', finishDate);
    }
    return <Observable<BeerConsumptionReport>>(
      this.http
        .get<BeerConsumptionReport>(url, options)
        .pipe(catchError(this.log.handleError(serviceName, 'get')))
    );
  }
}
