import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { Device } from '../core/device';
import { ErrorLoggerService } from '../core/error-logger.service';

const url = '/api/devices';
const serviceName = 'DeviceService';

@Injectable({
  providedIn: 'root',
})
export class DeviceService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string): Observable<Device> {
    const getUrl: string = id === null ? `${url}` : `${url}/${id}`;
    return <Observable<Device>>(
      this.http
        .get<Device>(getUrl)
        .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`)))
    );
  }

  list(): Observable<Device[]> {
    return <Observable<Device[]>>(
      this.http
        .get<Device[]>(`${url}/list`)
        .pipe(catchError(this.log.handleError(serviceName, 'list')))
    );
  }

  save(device: Device): Observable<Device> {
    return <Observable<Device>>(
      this.http
        .post<Device>(`${url}`, device)
        .pipe(catchError(this.log.handleError(serviceName, 'save')))
    );
  }

  update(device: Device): Observable<Device> {
    return <Observable<Device>>(
      this.http
        .put<Device>(`${url}/${device.id}`, device)
        .pipe(catchError(this.log.handleError(serviceName, 'update')))
    );
  }

  saveOrUpdate(device: Device): Observable<Device> {
    if (!device.id) {
      return this.save(device);
    }
    return this.update(device);
  }

  delete(id: string): Observable<Device> {
    return <Observable<Device>>(
      this.http
        .delete<Device>(`${url}/${id}`)
        .pipe(catchError(this.log.handleError(serviceName, 'delete')))
    );
  }
}
